@@ -1,39 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
+**
 ** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,6 +43,7 @@
 #define QDECLARATIVENOTIFIER_P_H
 
 #include "private/qdeclarativeguard_p.h"
+#include <QtCore/qmetaobject.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -77,8 +78,13 @@ public:
 
     void connect(QObject *source, int sourceSignal);
     inline void connect(QDeclarativeNotifier *);
+
+    // Disconnects unconditionally, regardless of the refcount
     inline void disconnect();
 
+    // Decreases the refcount and disconnects when refcount reaches 0
+    inline void deref();
+
     void copyAndClear(QDeclarativeNotifierEndpoint &other);
 
 private:
@@ -110,6 +116,8 @@ private:
         Notifier notifier;
     };
 
+    quint16 refCount;
+
     inline Notifier *toNotifier();
     inline Notifier *asNotifier();
     inline Signal *toSignal();
@@ -143,12 +151,12 @@ void QDeclarativeNotifier::notify()
 }
 
 QDeclarativeNotifierEndpoint::QDeclarativeNotifierEndpoint()
-: target(0), targetMethod(0), type(InvalidType) 
+    : target(0), targetMethod(0), type(InvalidType), refCount(0)
 {
 }
 
 QDeclarativeNotifierEndpoint::QDeclarativeNotifierEndpoint(QObject *t, int m)
-: target(t), targetMethod(m), type(InvalidType) 
+: target(t), targetMethod(m), type(InvalidType), refCount(0)
 {
 }
 
@@ -186,8 +194,10 @@ void QDeclarativeNotifierEndpoint::connect(QDeclarativeNotifier *notifier)
 {
     Notifier *n = toNotifier();
     
-    if (n->notifier == notifier)
+    if (n->notifier == notifier) {
+        refCount++;
         return;
+    }
 
     disconnect();
 
@@ -196,6 +206,7 @@ void QDeclarativeNotifierEndpoint::connect(QDeclarativeNotifier *notifier)
     notifier->endpoints = this;
     n->prev = &notifier->endpoints;
     n->notifier = notifier;
+    refCount++;
 }
 
 void QDeclarativeNotifierEndpoint::disconnect()
@@ -204,6 +215,11 @@ void QDeclarativeNotifierEndpoint::disconnect()
         Signal *s = asSignal();
         if (s->source) {
             QMetaObject::disconnectOne(s->source, s->sourceSignal, target, targetMethod);
+            QObjectPrivate * const priv = QObjectPrivate::get(s->source);
+            const QMetaMethod signal = s->source->metaObject()->method(s->sourceSignal);
+            QVarLengthArray<char> signalSignature;
+            QObjectPrivate::signalSignature(signal, &signalSignature);
+            priv->disconnectNotify(signalSignature.constData());
             s->source = 0;
         }
     } else if (type == NotifierType) {
@@ -217,6 +233,14 @@ void QDeclarativeNotifierEndpoint::disconnect()
         n->disconnected = 0;
         n->notifier = 0;
     }
+    refCount = 0;
+}
+
+void QDeclarativeNotifierEndpoint::deref()
+{
+    refCount--;
+    if (refCount <= 0)
+        disconnect();
 }
 
 QDeclarativeNotifierEndpoint::Notifier *QDeclarativeNotifierEndpoint::toNotifier()
