@@ -1,39 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
+**
 ** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -54,6 +54,7 @@
 #include <qstringlist.h>
 #include <qvarlengtharray.h>
 #include <qvector.h>
+#include <qmath.h>
 #include <QDebug>
 #include <QSqlQuery>
 
@@ -115,7 +116,7 @@ class QODBCDriverPrivate
 public:
     enum DefaultCase{Lower, Mixed, Upper, Sensitive};
     QODBCDriverPrivate()
-    : hEnv(0), hDbc(0), unicode(false), useSchema(false), disconnectCount(0), isMySqlServer(false),
+    : hEnv(0), hDbc(0), unicode(false), useSchema(false), disconnectCount(0), datetime_precision(19), isMySqlServer(false),
            isMSSqlServer(false), isFreeTDSDriver(false), hasSQLFetchScroll(true),
            hasMultiResultSets(false), isQuoteInitialized(false), quote(QLatin1Char('"'))
     {
@@ -127,6 +128,7 @@ public:
     bool unicode;
     bool useSchema;
     int disconnectCount;
+    int datetime_precision;
     bool isMySqlServer;
     bool isMSSqlServer;
     bool isFreeTDSDriver;
@@ -139,6 +141,7 @@ public:
     void checkHasSQLFetchScroll();
     void checkHasMultiResults();
     void checkSchemaUsage();
+    void checkDateTimePrecision();
     bool setConnectionOptions(const QString& connOpts);
     void splitTableQualifier(const QString &qualifier, QString &catalog,
                              QString &schema, QString &table);
@@ -963,13 +966,8 @@ bool QODBCResult::reset (const QString& query)
         return false;
     }
 
-    if(r == SQL_NO_DATA) {
-        setSelect(false);
-        return true;
-    }
-
-    SQLINTEGER isScrollable, bufferLength;
-    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, &bufferLength);
+    SQLULEN isScrollable = 0;
+    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, 0);
     if(r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO)
         QSqlResult::setForwardOnly(isScrollable==SQL_NONSCROLLABLE);
 
@@ -1115,7 +1113,7 @@ bool QODBCResult::fetchLast()
                 "Unable to fetch last"), QSqlError::ConnectionError, d));
         return false;
     }
-    SQLINTEGER currRow;
+    SQLULEN currRow = 0;
     r = SQLGetStmtAttr(d->hStmt,
                         SQL_ROW_NUMBER,
                         &currRow,
@@ -1333,12 +1331,12 @@ bool QODBCResult::exec()
     if (isSelect())
         SQLCloseCursor(d->hStmt);
 
-    QList<QByteArray> tmpStorage; // holds temporary buffers
-    QVarLengthArray<SQLLEN, 32> indicators(boundValues().count());
+    QVector<QVariant>& values = boundValues();
+    QVector<QByteArray> tmpStorage(values.count(), QByteArray()); // holds temporary buffers
+    QVarLengthArray<SQLLEN, 32> indicators(values.count());
     memset(indicators.data(), 0, indicators.size() * sizeof(SQLLEN));
 
     // bind parameters - only positional binding allowed
-    QVector<QVariant>& values = boundValues();
     int i;
     SQLRETURN r;
     for (i = 0; i < values.count(); ++i) {
@@ -1350,7 +1348,7 @@ bool QODBCResult::exec()
             *ind = SQL_NULL_DATA;
         switch (val.type()) {
             case QVariant::Date: {
-                QByteArray ba;
+                QByteArray &ba = tmpStorage[i];
                 ba.resize(sizeof(DATE_STRUCT));
                 DATE_STRUCT *dt = (DATE_STRUCT *)ba.constData();
                 QDate qdt = val.toDate();
@@ -1367,10 +1365,9 @@ bool QODBCResult::exec()
                                       (void *) dt,
                                       0,
                                       *ind == SQL_NULL_DATA ? ind : NULL);
-                tmpStorage.append(ba);
                 break; }
             case QVariant::Time: {
-                QByteArray ba;
+                QByteArray &ba = tmpStorage[i];
                 ba.resize(sizeof(TIME_STRUCT));
                 TIME_STRUCT *dt = (TIME_STRUCT *)ba.constData();
                 QTime qdt = val.toTime();
@@ -1387,10 +1384,9 @@ bool QODBCResult::exec()
                                       (void *) dt,
                                       0,
                                       *ind == SQL_NULL_DATA ? ind : NULL);
-                tmpStorage.append(ba);
                 break; }
             case QVariant::DateTime: {
-                QByteArray ba;
+                QByteArray &ba = tmpStorage[i];
                 ba.resize(sizeof(TIMESTAMP_STRUCT));
                 TIMESTAMP_STRUCT * dt = (TIMESTAMP_STRUCT *)ba.constData();
                 QDateTime qdt = val.toDateTime();
@@ -1400,18 +1396,28 @@ bool QODBCResult::exec()
                 dt->hour = qdt.time().hour();
                 dt->minute = qdt.time().minute();
                 dt->second = qdt.time().second();
-                dt->fraction = qdt.time().msec() * 1000000;
+
+                int precision = d->driverPrivate->datetime_precision - 20; // (20 includes a separating period)
+                if (precision <= 0) {
+                    dt->fraction = 0;
+                } else {
+                    dt->fraction = qdt.time().msec() * 1000000;
+
+                    // (How many leading digits do we want to keep?  With SQL Server 2005, this should be 3: 123000000)
+                    int keep = (int)qPow(10.0, 9 - qMin(9, precision));
+                    dt->fraction = (dt->fraction / keep) * keep;
+                }
+
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
                                       qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
                                       SQL_C_TIMESTAMP,
                                       SQL_TIMESTAMP,
-                                      19,
-                                      0,
+                                      d->driverPrivate->datetime_precision,
+                                      precision,
                                       (void *) dt,
                                       0,
                                       *ind == SQL_NULL_DATA ? ind : NULL);
-                tmpStorage.append(ba);
                 break; }
             case QVariant::Int:
                 r = SQLBindParameter(d->hStmt,
@@ -1502,14 +1508,15 @@ bool QODBCResult::exec()
                 break;
             case QVariant::String:
                 if (d->unicode) {
+                    QByteArray &ba = tmpStorage[i];
                     QString str = val.toString();
                     if (*ind != SQL_NULL_DATA)
                         *ind = str.length() * sizeof(SQLTCHAR);
                     int strSize = str.length() * sizeof(SQLTCHAR);
 
                     if (bindValueType(i) & QSql::Out) {
-                        QVarLengthArray<SQLTCHAR> ba(toSQLTCHAR(str));
-                        ba.reserve(str.capacity());
+                        QVarLengthArray<SQLTCHAR> a(toSQLTCHAR(str));
+                        a.reserve(str.capacity());
                         r = SQLBindParameter(d->hStmt,
                                             i + 1,
                                             qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
@@ -1517,13 +1524,12 @@ bool QODBCResult::exec()
                                             strSize > 254 ? SQL_WLONGVARCHAR : SQL_WVARCHAR,
                                             0, // god knows... don't change this!
                                             0,
-                                            (void *)ba.constData(),
-                                            ba.size(),
+                                            (void *)a.constData(),
+                                            a.size(),
                                             ind);
-                        tmpStorage.append(QByteArray((const char *)ba.constData(), ba.size()*sizeof(SQLTCHAR)));
                         break;
                     }
-                    QByteArray strba((const char *)toSQLTCHAR(str).constData(), str.size()*sizeof(SQLTCHAR));
+                    ba = QByteArray((const char *)toSQLTCHAR(str).constData(), str.size()*sizeof(SQLTCHAR));
                     r = SQLBindParameter(d->hStmt,
                                           i + 1,
                                           qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
@@ -1531,15 +1537,15 @@ bool QODBCResult::exec()
                                           strSize > 254 ? SQL_WLONGVARCHAR : SQL_WVARCHAR,
                                           strSize,
                                           0,
-                                          (SQLPOINTER)strba.constData(),
-                                          strba.size(),
+                                          (SQLPOINTER)ba.constData(),
+                                          ba.size(),
                                           ind);
-                    tmpStorage.append(strba);
                     break;
                 }
                 else
                 {
-                    QByteArray str = val.toString().toUtf8();
+                    QByteArray &str = tmpStorage[i];
+                    str = val.toString().toUtf8();
                     if (*ind != SQL_NULL_DATA)
                         *ind = str.length();
                     int strSize = str.length();
@@ -1554,12 +1560,11 @@ bool QODBCResult::exec()
                                           (void *)str.constData(),
                                           strSize,
                                           ind);
-                    tmpStorage.append(str);
                     break;
                 }
             // fall through
             default: {
-                QByteArray ba = val.toByteArray();
+                QByteArray &ba = tmpStorage[i];
                 if (*ind != SQL_NULL_DATA)
                     *ind = ba.size();
                 r = SQLBindParameter(d->hStmt,
@@ -1572,7 +1577,6 @@ bool QODBCResult::exec()
                                       (void *) ba.constData(),
                                       ba.length() + 1,
                                       ind);
-                tmpStorage.append(ba);
                 break; }
         }
         if (r != SQL_SUCCESS) {
@@ -1583,7 +1587,7 @@ bool QODBCResult::exec()
         }
     }
     r = SQLExecute(d->hStmt);
-    if (r != SQL_SUCCESS && r != SQL_SUCCESS_WITH_INFO) {
+    if (r != SQL_SUCCESS && r != SQL_SUCCESS_WITH_INFO && r != SQL_NO_DATA) {
         qWarning() << "QODBCResult::exec: Unable to execute statement:" << qODBCWarn(d);
         setLastError(qMakeError(QCoreApplication::translate("QODBCResult",
                      "Unable to execute statement"), QSqlError::StatementError, d));
@@ -1591,8 +1595,7 @@ bool QODBCResult::exec()
     }
 
     SQLULEN isScrollable = 0;
-    SQLINTEGER bufferLength;
-    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, &bufferLength);
+    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, 0);
     if(r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO)
         QSqlResult::setForwardOnly(isScrollable==SQL_NONSCROLLABLE);
 
@@ -1617,16 +1620,16 @@ bool QODBCResult::exec()
     for (i = 0; i < values.count(); ++i) {
         switch (values.at(i).type()) {
             case QVariant::Date: {
-                DATE_STRUCT ds = *((DATE_STRUCT *)tmpStorage.takeFirst().constData());
+                DATE_STRUCT ds = *((DATE_STRUCT *)tmpStorage.at(i).constData());
                 values[i] = QVariant(QDate(ds.year, ds.month, ds.day));
                 break; }
             case QVariant::Time: {
-                TIME_STRUCT dt = *((TIME_STRUCT *)tmpStorage.takeFirst().constData());
+                TIME_STRUCT dt = *((TIME_STRUCT *)tmpStorage.at(i).constData());
                 values[i] = QVariant(QTime(dt.hour, dt.minute, dt.second));
                 break; }
             case QVariant::DateTime: {
                 TIMESTAMP_STRUCT dt = *((TIMESTAMP_STRUCT*)
-                                        tmpStorage.takeFirst().constData());
+                                        tmpStorage.at(i).constData());
                 values[i] = QVariant(QDateTime(QDate(dt.year, dt.month, dt.day),
                                QTime(dt.hour, dt.minute, dt.second, dt.fraction / 1000000)));
                 break; }
@@ -1642,7 +1645,7 @@ bool QODBCResult::exec()
             case QVariant::String:
                 if (d->unicode) {
                     if (bindValueType(i) & QSql::Out) {
-                        QByteArray first = tmpStorage.takeFirst();
+                        const QByteArray &first = tmpStorage.at(i);
                         QVarLengthArray<SQLTCHAR> array;
                         array.append((SQLTCHAR *)first.constData(), first.size());
                         values[i] = fromSQLTCHAR(array, first.size()/sizeof(SQLTCHAR*));
@@ -1652,7 +1655,7 @@ bool QODBCResult::exec()
                 // fall through
             default: {
                 if (bindValueType(i) & QSql::Out)
-                    values[i] = tmpStorage.takeFirst();
+                    values[i] = tmpStorage.at(i);
                 break; }
         }
         if (indicators[i] == SQL_NULL_DATA)
@@ -1894,6 +1897,7 @@ bool QODBCDriver::open(const QString & db,
     d->checkSqlServer();
     d->checkHasSQLFetchScroll();
     d->checkHasMultiResults();
+    d->checkDateTimePrecision();
     setOpen(true);
     setOpenError(false);
     if(d->isMSSqlServer) {
@@ -2129,6 +2133,29 @@ void QODBCDriverPrivate::checkHasMultiResults()
 #endif
 }
 
+void QODBCDriverPrivate::checkDateTimePrecision()
+{
+    SQLINTEGER columnSize;
+    SQLHANDLE hStmt;
+
+    SQLRETURN r = SQLAllocHandle(SQL_HANDLE_STMT, hDbc, &hStmt);
+    if (r != SQL_SUCCESS) {
+        return;
+    }
+
+    r = SQLGetTypeInfo(hStmt, SQL_TIMESTAMP);
+    if (r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO) {
+        r = SQLFetch(hStmt);
+        if ( r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO )
+        {
+            if (SQLGetData(hStmt, 3, SQL_INTEGER, &columnSize, sizeof(columnSize), 0) == SQL_SUCCESS) {
+                datetime_precision = (int)columnSize;
+            }
+        }
+    }
+    SQLFreeHandle(SQL_HANDLE_STMT, hStmt);
+}
+
 QSqlResult *QODBCDriver::createResult() const
 {
     return new QODBCResult(this, d);
